/*
 * Copyright (c) 2013 by Piero Orsoni (www.opbyte.it)
 * Based on the "sash" project by David I. Bell
 *
 * This file is part of cbmsh, a unix-like shell for commodore computers.
 *
 * Cbmsh is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cbmsh is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with cbmsh.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "cbmsh.h"

//#include <sys/types.h>
//#include <sys/stat.h>
//#include <utime.h>
#include <errno.h>
#include <conio.h>
#include <device.h>


void do_echo(int argc, const char ** argv) {
	BOOL first = TRUE;
	while (argc-- > 1) {
		if (!first) putchar(' ');
		first = FALSE;
		printf(*(++argv));
	}
	putchar('\n');
}


	/*
	const char *path = "/";
	char buf[PATH_LEN];
	if (getcwd(buf, PATH_LEN) == NULL) {
		printf("Can't get dir\n");
		return;
	}
	printf("%s\n", buf);
	*/
/*
void do_pwd(int argc, const char ** argv) {
	printf("not supported\n");
}
*/


void do_cd(int argc, const char **argv) {
	char path[PATH_LEN + 3] = "cd:/";
	if (argc > 1) strncpy(&path[3], argv[1], PATH_LEN);
	cbm_command(path);
}


void do_mkdir(int argc, const char **argv) {
	char path[PATH_LEN + 3] = "md:";
	while (argc-- > 1) {
		//if (mkdir(argv[1], 0777) < 0) perror(argv[1]);
		strncpy(&path[3], argv[1], PATH_LEN);
		cbm_command(path);
		argv++;
	}
}


void do_rmdir(int argc, const char ** argv) {
	char path[PATH_LEN + 3] = "rd:";
	while (argc-- > 1) {
		strncpy(&path[3], argv[1], PATH_LEN);
		cbm_command(path);
		argv++;
	}
}


void do_rm(int argc, const char ** argv) {
	while (argc-- > 1) {
		if (remove(argv[1]) < 0) perror(argv[1]);
		argv++;
	}
}


void do_mv(int argc, const char ** argv) {
	const char *srcName;
	const char *destName;
	const char *lastArg = argv[argc - 1];

	/*
	BOOL dirFlag = isDirectory(lastArg);
	if ((argc > 3) && !dirFlag) {
		printf("%s not a dir\n", lastArg);
		return;
	}
	*/

	while (argc-- > 2) {
		srcName = *(++argv);
/*
		if (access(srcName, 0) < 0) {
			perror(srcName);
			continue;
		}
*/
		destName = lastArg;
//		if (dirFlag) destName = buildName(destName, srcName);
		if (rename(srcName, destName) >= 0) continue;
/*
		if (errno != EXDEV) {
			perror(destName);
			continue;
		}
*/
		if (!copyFile(srcName, destName)) continue;		//, TRUE
		if (remove(srcName) < 0) perror(srcName);
	}
}


void do_cp(int argc, const char ** argv) {
	const char *srcName;
	const char *destName;
	const char *lastArg = argv[argc - 1];
	/*
	BOOL dirFlag = isDirectory(lastArg);
	if ((argc > 3) && !dirFlag) {
		printf("%s not a dir\n", lastArg);
		return;
	}
	*/

	while (argc-- > 2) {
		srcName = *(++argv);
		destName = lastArg;
//		if (dirFlag) destName = buildName(destName, srcName);
		copyFile(srcName, destName);		//, FALSE
	}
}


#ifndef CBMSH_MINIMAL
void do_cmp(int argc, const char **argv) {
	int fd1, fd2;
	int cc1, cc2;
	char buf1[BUF_SIZE], buf2[BUF_SIZE];
	unsigned int pos = 0;
//	const char *bp1, *bp2;

	fd1 = open(argv[1], O_RDONLY);
	if (fd1 < 0) {
		perror(argv[1]);
		return;
	}

	fd2 = open(argv[2], O_RDONLY);
	if (fd2 < 0) {
		perror(argv[2]);
		close(fd1);
		return;
	}

	while (TRUE) {
//		if (intFlag) goto closefiles;
		cc1 = read(fd1, buf1, sizeof(buf1));
		if (cc1 < 0) {
			perror(argv[1]);
			break;
		}

		cc2 = read(fd2, buf2, sizeof(buf2));
		if (cc2 < 0) {
			perror(argv[2]);
			break;
		}

		if (cc1 == 0 && cc2 == 0)	{
			//printf("Identical\n");
			break;
		}

		if (cc1 < cc2) {
			printf("1 shorter\n");
			break;
		}

		if (cc1 > cc2) {
			printf("2 shorter\n");
			break;
		}

		if (memcmp(buf1, buf2, cc1) != 0) {
			pos += cc1;
			continue;
		}
/*
		bp1 = buf1;
		bp2 = buf2;
		while (*bp1++ == *bp2++) pos++;
		printf("Diff at %ld\n", pos);
*/
		printf("Differ\n");
		break;
	}

	close(fd1);
	close(fd2);
}


void do_sum(int argc, const char ** argv) {
	const char *name;
	int fd, cc, i;
	unsigned long	checksum;
	char buf[BUF_SIZE];

	argc--;
	argv++;

	while (argc-- > 0) {
		name = *argv++;
		fd = open(name, O_RDONLY);

		if (fd < 0) {
			perror(name);
			continue;
		}
		checksum = 0;

		while ((cc = read(fd, buf, sizeof(buf))) > 0) {
			for (i = 0; i < cc; i++) {
				if ((checksum & 0x01) != 0) checksum = (checksum >> 1) + 0x8000;
				else checksum = (checksum >> 1);
				checksum = (checksum + buf[i]) & 0xffff;
			}
		}

		if (cc < 0) {
			perror(name);
			(void) close(fd);
			continue;
		}

		(void) close(fd);
		printf("%05lu %s\n", checksum, name);
	}
}
#endif


void do_unit(int argc, const char ** argv) {
	char u, r;
//	char buf[3];
	struct cbm_dirent dp;

	if (argc == 1) {
		u = getfirstdevice();
		while (u != INVALID_DEVICE) {
//			revers(1);
			printf("%u", u);
			if (cbm_curunit() == u) putchar('*');
			else putchar(' ');
			if (u < 10) putchar(' ');

			if (cbm_opendir(14, u) == 0) {
				while ((r = cbm_readdir(14, &dp)) == 0) {
					if (dp.type == CBM_T_HEADER) {
						putchar(' ');
						printf(dp.name);
					}
				}
				if (r == 2) printf("\n%li free", (long)dp.size * 256);
				cbm_closedir(14);
			}

			/*
			if (getdevicedir(u, buf, 3)) {
				printf(" load");
				if (cbm_curunit() == u) printf(" curr");
			}
			*/
      putchar('\n');
			u = getnextdevice(u);
		}
	} else {
		if (chdir(argv[1]) < 0) perror(argv[1]);
	}
}


void do_more(int argc, const char ** argv) {
	FILE *fp;
	const char *name;
	char	pageLines, pageColumns;
	char	ch, tmp, prevch;
	char	line = 1, col = 0;
	// char	buf[80];

	//	BUG: sometimes does an additional CR after printing -more-, looks like it's related to lines not ending in \n

	screensize(&pageColumns, &pageLines);

	while (argc-- > 1) {
		name = *(++argv);
		fp = fopen(name, "r");

		if (fp == NULL) {
			perror(name);
			return;
		}

		printf("<%s>\n", name);
		line++;
//		line = 1;
//		col = 0;
		prevch = 0;

		while (fp && ((ch = fgetc(fp)) != 255)) {
			switch (ch) {
				case '\n':
				case '\r':
					break;
				case '\t':
					for (tmp = (col | 0x07) + 1; col < tmp; col++) putchar(' ');
					break;
				case '\b':
					if (col > 0) col--;
					break;
				default:
					col++;
			}

			if (ch == '\r' || ch == '\n') {
				if (prevch != '\n' || ch != '\r') {
					putchar('\n');
					line++;
					col = 0;
				}
			} else {
				putchar(ch);
			}
			prevch = ch;

			if (col >= pageColumns) {
				col -= pageColumns;
				line++;
			}
			if (line < pageLines) continue;
			//if (col > 0) putchar('\n');
			printf(STR_MORE);

			/*
			fflush(stdout);
			if ((read(stdin, buf, sizeof(buf)) < 0)) {		//intFlag ||
				if (fp) fclose(fp);
				return;
			}
			ch = buf[0];
			if (ch == ':') ch = buf[1];
			*/

			ch = cgetc();
			putchar('\n');

			switch (ch) {
				case 'N':
				case 'n':
					fclose(fp);
					fp = NULL;
					break;
				case 'Q':
				case 'q':
					fclose(fp);
					return;
			}

			col = 0;
			line = 2;
		}
		if (fp) fclose(fp);
	}
}


void do_exit(int argc, const char ** argv) {
	// free: (alias, list, fileTable, strings, argTable)?
	freeChunks();
	exit(0);
}


void do_clear(int argc, const char ** argv) {
	clrscr();
}


void do_dos(int argc, const char **argv) {
	while (argc-- > 1) {
		cbm_command((char*)argv[1]);
		argv++;
	}
}
